#ifndef _config_h_
#define _config_h_

/*HEADER**********************************************************************
*
* Copyright 2008 Freescale Semiconductor, Inc.
* Copyright 1989-2008 ARC International
*
* This software is owned or controlled by Freescale Semiconductor.
* Use of this software is governed by the Freescale MQX RTOS License
* distributed with this Material.
* See the MQX_RTOS_LICENSE file distributed for more details.
*
* Brief License Summary:
* This software is provided in source form for you to use free of charge,
* but it is not open source software. You are allowed to use this software
* but you cannot redistribute it or derivative works of it in source form.
* The software may be used only in connection with a product containing
* a Freescale microprocessor, microcontroller, or digital signal processor.
* See license agreement file for full license terms including other
* restrictions.
*****************************************************************************
*
* Comments:
*
*   Configurable information for the RTCS examples.
*
*
*END************************************************************************/

#include "httpsrv.h"
#include "rtcs_ssl.h"

/* IP address macros */
#define ENET_IPADDR_A 192
#define ENET_IPADDR_B 168
#define ENET_IPADDR_C 0
#define ENET_IPADDR_D 90

#define HTTPSRV_EXAMPLE_CGI_STACK (3000)
#define HTTPSRV_EXAMPLE_SESSIONS  (2)

/* Use this define to tell example if only one server should be used for all interfaces */
#define HTTP_USE_ONE_SERVER 1

#ifndef ENET_IPMASK
    #define ENET_IPMASK  IPADDR(255,255,255,0) 
#endif

#if RTCSCFG_ENABLE_IP6   
    #if RTCSCFG_ENABLE_IP4   
        /* 
        **  HTTP_INET_AF is AF_INET+AF_INET for http support IPv4+IPv6.
        **  HTTP_INET_AF is AF_INET  for http support IPv4 only.
        **  HTTP_INET_AF is AF_INET6 for http support IPv6 only.
        */
        #define HTTP_INET_AF    (AF_INET | AF_INET6)
    #else
        #define HTTP_INET_AF    AF_INET6
    #endif
#else
    #define HTTP_INET_AF    AF_INET
#endif

#define HTTP_SCOPE_ID          0 /* For any IF. */

#if ENABLE_SSL
    #define SSL_CERT_PATH "tfs:\\ssl\\server_cert.pem"
    #define SSL_KEY_PATH "tfs:\\ssl\\server_key.pem"
    #define HTTPSRV_PORT (443)
    static const RTCS_SSL_PARAMS_STRUCT ssl_params = {
        SSL_CERT_PATH,
        SSL_KEY_PATH,
        NULL,
        RTCS_SSL_SERVER
    };    
#else
    #define HTTPSRV_PORT (80)
    static const RTCS_SSL_PARAMS_STRUCT ssl_params = {
        NULL,
        NULL,
        NULL,
        RTCS_SSL_SERVER
    };    
#endif
extern HTTPSRV_PLUGIN_LINK_STRUCT plugins;
extern uint32_t init_plugins(void);

/*
 * Authentication users
 */
HTTPSRV_AUTH_USER_STRUCT users[] = {
    {"developer", "freescale"},
    { NULL, NULL} /* Array terminator */
};

/*
 * Authentication information.
 */
HTTPSRV_AUTH_REALM_STRUCT auth_realms[] = {
    { "Please use uid:developer pass:freescale to login", "auth.html", HTTPSRV_AUTH_BASIC, users},
    { NULL, NULL, HTTPSRV_AUTH_INVALID, NULL} /* Array terminator */
};

#endif // _config_h_
